"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_router_1 = require("react-router");
const buildRawPath = (map, name) => {
    const route = map.get(name);
    if (!route) {
        throw new Error(`Undefined route "${name}"`);
    }
    if (!route.path) {
        throw new Error(`Route "${name}" does not have a path`);
    }
    return route.path;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
exports.buildRoutePath = (map, name, params = {}) => (buildRawPath(map, name)
    .replace(/:\w+\??/g, (match) => {
    const paramKey = match.replace(/[:?]/g, '');
    const paramValue = params[paramKey];
    if (!paramValue && !match.includes('?')) {
        throw new Error(`Missing value for required param "${paramKey}"`);
    }
    return paramValue || '';
})
    .replace(/\/$/, ''));
exports.mapRoutes = (routes, basename = '', parents = [], map = new Map()) => {
    routes.forEach((route) => {
        if (route.routes) {
            exports.mapRoutes(route.routes, basename, route.name ? [...parents, route.name] : parents, map);
        }
        if (!route.name && route.path) {
            throw new Error(`Route ${route.path} has no name`);
        }
        if (!route.name) {
            return;
        }
        const existingRoute = map.get(route.name);
        if (process.env.NODE_ENV !== 'production' && existingRoute) {
            // eslint-disable-next-line no-console
            console.warn(`Duplicate definition for route ${route.name}:\n- ${existingRoute.path}\n- ${route.path}`);
        }
        map.set(route.name, Object.assign({}, route, { parents }, (route.path ? {
            regex: new RegExp(`^${(basename + route.path)
                .replace(/\/:\w+\?/g, '/?([^\\/]*)')
                .replace(/:\w+/g, '([^\\/]+)')}\\/?${route.exact ? '$' : ''}`),
        } : {})));
    });
    return map;
};
const omitName = (_a) => {
    var { name } = _a, rest = __rest(_a, ["name"]);
    return rest;
};
const defaultParams = {};
class RoutingContext {
    constructor(routesMap, routerContext) {
        this.match = (pathname, matchAll) => Array.from(this.routesMap.values())
            .find(route => (matchAll || (!matchAll && route.exact)) && !!route.regex && route.regex.test(pathname)) || null;
        this.getPath = (name, params) => exports.buildRoutePath(this.routesMap, name, params);
        this.getRawPath = (name) => buildRawPath(this.routesMap, name);
        this.getRoute = (name) => {
            const route = this.routesMap.get(name);
            if (!route) {
                throw new Error(`Undefined route "${name}"`);
            }
            return route;
        };
        this.push = (location, params) => this.history.push(Object.assign({ pathname: exports.buildRoutePath(this.routesMap, typeof location === 'string' ? location : location.name, params) }, (typeof location === 'string' ? undefined : omitName(location))));
        this.replace = (location, params) => this.history.replace(Object.assign({ pathname: exports.buildRoutePath(this.routesMap, typeof location === 'string' ? location : location.name, params) }, (typeof location === 'string' ? undefined : omitName(location))));
        this.routesMap = routesMap;
        this.routerContext = routerContext;
    }
    get location() {
        return this.routerContext.location;
    }
    get history() {
        if (!this.routerContext.history) {
            throw new Error('History API cannot be used on server side');
        }
        return this.routerContext.history;
    }
    get params() {
        const route = this.match(this.location.pathname);
        if (!route || !route.path) {
            return defaultParams;
        }
        const match = react_router_1.matchPath(this.location.pathname, route);
        return match ? match.params : defaultParams;
    }
    get currentRoute() {
        return this.match(this.location.pathname);
    }
}
exports.RoutingContext = RoutingContext;
exports.buildRoutingContext = (routes, routerContext, basename) => {
    const map = exports.mapRoutes(routes, basename);
    let finalContext = routerContext;
    if (typeof routerContext.location === 'string') {
        const [pathname, search] = routerContext.location.split('?');
        finalContext = Object.assign({}, routerContext, { location: {
                pathname,
                search: search ? `?${search}` : '',
                state: {},
                hash: '',
            } });
    }
    return new RoutingContext(map, finalContext);
};
